@@ -210,6 +210,7 @@ typedef enum status_rc
   STATUS_ABORTED_RUNTIME    = 11,
   STATUS_ERROR              = 13,
   STATUS_ABORTED_FINISH     = 14,
+  STATUS_AUTODETECT         = 16,
 
 } status_rc_t;
 
@@ -442,6 +443,8 @@ typedef enum opts_type
   OPTS_TYPE_SELF_TEST_DISABLE = (1ULL << 51), // some algos use JiT in combinations with a salt or create too much startup time
   OPTS_TYPE_MP_MULTI_DISABLE  = (1ULL << 52), // do not multiply the kernel-accel with the multiprocessor count per device to allow more fine-tuned workload settings
   OPTS_TYPE_NATIVE_THREADS    = (1ULL << 53), // forces "native" thread count: CPU=1, GPU-Intel=8, GPU-AMD=64 (wavefront), GPU-NV=32 (warps)
+  OPTS_TYPE_POST_AMP_UTF16LE  = (1ULL << 54), // run the utf8 to utf16le conversion kernel after they have been processed from amplifiers
+  OPTS_TYPE_AUTODETECT_DISABLE = (1ULL << 55), // skip autodetect engine
 
 } opts_type_t;
 
@@ -554,6 +557,8 @@ typedef enum parser_rc
   PARSER_FILE_SIZE            = -41,
   PARSER_IV_LENGTH            = -42,
   PARSER_CT_LENGTH            = -43,
+  PARSER_CRYPTOAPI_KERNELTYPE = -44,
+  PARSER_CRYPTOAPI_KEYSIZE    = -45,
   PARSER_HAVE_ERRNO           = -100,
   PARSER_UNKNOWN_ERROR        = -255
 
@@ -591,6 +596,7 @@ typedef enum user_options_defaults
 {
   ADVICE_DISABLE           = false,
   ATTACK_MODE              = ATTACK_MODE_STRAIGHT,
+  AUTODETECT               = false,
   BENCHMARK_ALL            = false,
   BENCHMARK                = false,
   BITMAP_MAX               = 18,
@@ -613,6 +619,7 @@ typedef enum user_options_defaults
   HEX_SALT                 = false,
   HEX_WORDLIST             = false,
   HOOK_THREADS             = 0,
+  IDENTIFY                 = false,
   INCREMENT                = false,
   INCREMENT_MAX            = PW_MAX,
   INCREMENT_MIN            = 1,
@@ -719,73 +726,74 @@ typedef enum user_options_map
   IDX_HEX_SALT                  = 0xff1a,
   IDX_HEX_WORDLIST              = 0xff1b,
   IDX_HOOK_THREADS              = 0xff1c,
+  IDX_IDENTIFY                  = 0xff1d,
   IDX_INCREMENT                 = 'i',
-  IDX_INCREMENT_MAX             = 0xff1d,
-  IDX_INCREMENT_MIN             = 0xff1e,
-  IDX_INDUCTION_DIR             = 0xff1f,
-  IDX_KEEP_GUESSING             = 0xff20,
+  IDX_INCREMENT_MAX             = 0xff1e,
+  IDX_INCREMENT_MIN             = 0xff1f,
+  IDX_INDUCTION_DIR             = 0xff20,
+  IDX_KEEP_GUESSING             = 0xff21,
   IDX_KERNEL_ACCEL              = 'n',
   IDX_KERNEL_LOOPS              = 'u',
   IDX_KERNEL_THREADS            = 'T',
-  IDX_KEYBOARD_LAYOUT_MAPPING   = 0xff21,
-  IDX_KEYSPACE                  = 0xff22,
-  IDX_LEFT                      = 0xff23,
+  IDX_KEYBOARD_LAYOUT_MAPPING   = 0xff22,
+  IDX_KEYSPACE                  = 0xff23,
+  IDX_LEFT                      = 0xff24,
   IDX_LIMIT                     = 'l',
-  IDX_LOGFILE_DISABLE           = 0xff24,
-  IDX_LOOPBACK                  = 0xff25,
-  IDX_MACHINE_READABLE          = 0xff26,
-  IDX_MARKOV_CLASSIC            = 0xff27,
-  IDX_MARKOV_DISABLE            = 0xff28,
-  IDX_MARKOV_HCSTAT2            = 0xff29,
+  IDX_LOGFILE_DISABLE           = 0xff25,
+  IDX_LOOPBACK                  = 0xff26,
+  IDX_MACHINE_READABLE          = 0xff27,
+  IDX_MARKOV_CLASSIC            = 0xff28,
+  IDX_MARKOV_DISABLE            = 0xff29,
+  IDX_MARKOV_HCSTAT2            = 0xff2a,
   IDX_MARKOV_THRESHOLD          = 't',
-  IDX_NONCE_ERROR_CORRECTIONS   = 0xff2a,
+  IDX_NONCE_ERROR_CORRECTIONS   = 0xff2b,
   IDX_OPENCL_DEVICE_TYPES       = 'D',
   IDX_OPTIMIZED_KERNEL_ENABLE   = 'O',
-  IDX_OUTFILE_AUTOHEX_DISABLE   = 0xff2b,
-  IDX_OUTFILE_CHECK_DIR         = 0xff2c,
-  IDX_OUTFILE_CHECK_TIMER       = 0xff2d,
-  IDX_OUTFILE_FORMAT            = 0xff2e,
+  IDX_OUTFILE_AUTOHEX_DISABLE   = 0xff2c,
+  IDX_OUTFILE_CHECK_DIR         = 0xff2d,
+  IDX_OUTFILE_CHECK_TIMER       = 0xff2e,
+  IDX_OUTFILE_FORMAT            = 0xff2f,
   IDX_OUTFILE                   = 'o',
-  IDX_POTFILE_DISABLE           = 0xff2f,
-  IDX_POTFILE_PATH              = 0xff30,
-  IDX_PROGRESS_ONLY             = 0xff31,
-  IDX_QUIET                     = 0xff32,
-  IDX_REMOVE                    = 0xff33,
-  IDX_REMOVE_TIMER              = 0xff34,
-  IDX_RESTORE                   = 0xff35,
-  IDX_RESTORE_DISABLE           = 0xff36,
-  IDX_RESTORE_FILE_PATH         = 0xff37,
+  IDX_POTFILE_DISABLE           = 0xff30,
+  IDX_POTFILE_PATH              = 0xff31,
+  IDX_PROGRESS_ONLY             = 0xff32,
+  IDX_QUIET                     = 0xff33,
+  IDX_REMOVE                    = 0xff34,
+  IDX_REMOVE_TIMER              = 0xff35,
+  IDX_RESTORE                   = 0xff36,
+  IDX_RESTORE_DISABLE           = 0xff37,
+  IDX_RESTORE_FILE_PATH         = 0xff38,
   IDX_RP_FILE                   = 'r',
-  IDX_RP_GEN_FUNC_MAX           = 0xff38,
-  IDX_RP_GEN_FUNC_MIN           = 0xff39,
+  IDX_RP_GEN_FUNC_MAX           = 0xff39,
+  IDX_RP_GEN_FUNC_MIN           = 0xff3a,
   IDX_RP_GEN                    = 'g',
-  IDX_RP_GEN_SEED               = 0xff3a,
+  IDX_RP_GEN_SEED               = 0xff3b,
   IDX_RULE_BUF_L                = 'j',
   IDX_RULE_BUF_R                = 'k',
-  IDX_RUNTIME                   = 0xff3b,
-  IDX_SCRYPT_TMTO               = 0xff3c,
+  IDX_RUNTIME                   = 0xff3c,
+  IDX_SCRYPT_TMTO               = 0xff3d,
   IDX_SEGMENT_SIZE              = 'c',
-  IDX_SELF_TEST_DISABLE         = 0xff3d,
+  IDX_SELF_TEST_DISABLE         = 0xff3e,
   IDX_SEPARATOR                 = 'p',
-  IDX_SESSION                   = 0xff3e,
-  IDX_SHOW                      = 0xff3f,
+  IDX_SESSION                   = 0xff3f,
+  IDX_SHOW                      = 0xff40,
   IDX_SKIP                      = 's',
   IDX_SLOW_CANDIDATES           = 'S',
-  IDX_SPEED_ONLY                = 0xff40,
-  IDX_SPIN_DAMP                 = 0xff41,
-  IDX_STATUS                    = 0xff42,
-  IDX_STATUS_JSON               = 0xff43,
-  IDX_STATUS_TIMER              = 0xff44,
-  IDX_STDOUT_FLAG               = 0xff45,
-  IDX_STDIN_TIMEOUT_ABORT       = 0xff46,
-  IDX_TRUECRYPT_KEYFILES        = 0xff47,
-  IDX_USERNAME                  = 0xff48,
-  IDX_VERACRYPT_KEYFILES        = 0xff49,
-  IDX_VERACRYPT_PIM_START       = 0xff4a,
-  IDX_VERACRYPT_PIM_STOP        = 0xff4b,
+  IDX_SPEED_ONLY                = 0xff41,
+  IDX_SPIN_DAMP                 = 0xff42,
+  IDX_STATUS                    = 0xff43,
+  IDX_STATUS_JSON               = 0xff44,
+  IDX_STATUS_TIMER              = 0xff45,
+  IDX_STDOUT_FLAG               = 0xff46,
+  IDX_STDIN_TIMEOUT_ABORT       = 0xff47,
+  IDX_TRUECRYPT_KEYFILES        = 0xff48,
+  IDX_USERNAME                  = 0xff49,
+  IDX_VERACRYPT_KEYFILES        = 0xff4a,
+  IDX_VERACRYPT_PIM_START       = 0xff4b,
+  IDX_VERACRYPT_PIM_STOP        = 0xff4c,
   IDX_VERSION_LOWER             = 'v',
   IDX_VERSION                   = 'V',
-  IDX_WORDLIST_AUTOHEX_DISABLE  = 0xff4c,
+  IDX_WORDLIST_AUTOHEX_DISABLE  = 0xff4d,
   IDX_WORKLOAD_PROFILE          = 'w',
 
 } user_options_map_t;
@@ -1114,6 +1122,7 @@ typedef struct hc_device_param
   u32     kernel_wgs_tm;
   u32     kernel_wgs_memset;
   u32     kernel_wgs_atinit;
+  u32     kernel_wgs_utf8toutf16le;
   u32     kernel_wgs_decompress;
   u32     kernel_wgs_aux1;
   u32     kernel_wgs_aux2;
@@ -1138,6 +1147,7 @@ typedef struct hc_device_param
   u32     kernel_preferred_wgs_multiple_tm;
   u32     kernel_preferred_wgs_multiple_memset;
   u32     kernel_preferred_wgs_multiple_atinit;
+  u32     kernel_preferred_wgs_multiple_utf8toutf16le;
   u32     kernel_preferred_wgs_multiple_decompress;
   u32     kernel_preferred_wgs_multiple_aux1;
   u32     kernel_preferred_wgs_multiple_aux2;
@@ -1162,6 +1172,7 @@ typedef struct hc_device_param
   u64     kernel_local_mem_size_tm;
   u64     kernel_local_mem_size_memset;
   u64     kernel_local_mem_size_atinit;
+  u64     kernel_local_mem_size_utf8toutf16le;
   u64     kernel_local_mem_size_decompress;
   u64     kernel_local_mem_size_aux1;
   u64     kernel_local_mem_size_aux2;
@@ -1186,6 +1197,7 @@ typedef struct hc_device_param
   u64     kernel_dynamic_local_mem_size_tm;
   u64     kernel_dynamic_local_mem_size_memset;
   u64     kernel_dynamic_local_mem_size_atinit;
+  u64     kernel_dynamic_local_mem_size_utf8toutf16le;
   u64     kernel_dynamic_local_mem_size_decompress;
   u64     kernel_dynamic_local_mem_size_aux1;
   u64     kernel_dynamic_local_mem_size_aux2;
@@ -1346,6 +1358,7 @@ typedef struct hc_device_param
   void   *kernel_params_tm[PARAMCNT];
   void   *kernel_params_memset[PARAMCNT];
   void   *kernel_params_atinit[PARAMCNT];
+  void   *kernel_params_utf8toutf16le[PARAMCNT];
   void   *kernel_params_decompress[PARAMCNT];
 
   u32     kernel_params_buf32[PARAMCNT];
@@ -1369,6 +1382,9 @@ typedef struct hc_device_param
   u32     kernel_params_atinit_buf32[PARAMCNT];
   u64     kernel_params_atinit_buf64[PARAMCNT];
 
+  u32     kernel_params_utf8toutf16le_buf32[PARAMCNT];
+  u64     kernel_params_utf8toutf16le_buf64[PARAMCNT];
+
   u32     kernel_params_decompress_buf32[PARAMCNT];
   u64     kernel_params_decompress_buf64[PARAMCNT];
 
@@ -1408,6 +1424,7 @@ typedef struct hc_device_param
   CUfunction        cuda_function_tm;
   CUfunction        cuda_function_memset;
   CUfunction        cuda_function_atinit;
+  CUfunction        cuda_function_utf8toutf16le;
   CUfunction        cuda_function_decompress;
   CUfunction        cuda_function_aux1;
   CUfunction        cuda_function_aux2;
@@ -1492,6 +1509,7 @@ typedef struct hc_device_param
   cl_kernel         opencl_kernel_tm;
   cl_kernel         opencl_kernel_memset;
   cl_kernel         opencl_kernel_atinit;
+  cl_kernel         opencl_kernel_utf8toutf16le;
   cl_kernel         opencl_kernel_decompress;
   cl_kernel         opencl_kernel_aux1;
   cl_kernel         opencl_kernel_aux2;
@@ -1926,6 +1944,7 @@ typedef struct user_options
   char       **hc_argv;
 
   bool         attack_mode_chgd;
+  bool         autodetect;
   #ifdef WITH_BRAIN
   bool         brain_host_chgd;
   bool         brain_port_chgd;
@@ -1934,6 +1953,7 @@ typedef struct user_options
   #endif
   bool         hash_mode_chgd;
   bool         hccapx_message_pair_chgd;
+  bool         identify;
   bool         increment_max_chgd;
   bool         increment_min_chgd;
   bool         kernel_accel_chgd;
@@ -2117,6 +2137,7 @@ typedef struct folder_config
   char *cwd;
   char *install_dir;
   char *profile_dir;
+  char *cache_dir;
   char *session_dir;
   char *shared_dir;
   char *cpath_real;
@@ -2662,7 +2683,9 @@ typedef enum hash_category
   HASH_CATEGORY_PLAIN                   = 18,
   HASH_CATEGORY_FRAMEWORK               = 19,
   HASH_CATEGORY_PRIVATE_KEY             = 20,
-
+  HASH_CATEGORY_IMS                     = 21,
+  HASH_CATEGORY_CRYPTOCURRENCY_WALLET   = 22,
+  HASH_CATEGORY_FBE                     = 23
 } hash_category_t;
 
 // hash specific
